

#include <stdlib.h>
#include <stdio.h>
#include "descartes.h"

/*
 *  Eulidean travelling salesman
 */

#define MAXCITIES 100
#define FALSE 0
#define TRUE 1

point_t city [MAXCITIES];
int numCities = 0;

int ReadCities(void) {
/* To be completed for Part A */
  point_t p;
  p=GetPoint();
  while ((XCoord(p) >= 0) && (numCities< MAXCITIES)) {
    city[numCities]=p;
    p=GetPoint();
    numCities++;
  }
  if (XCoord(p) <0)
    return TRUE;
  else 
    return FALSE;
}


double TourLength(void) {
/* To be completed for Part A */
  double len = 0.0;
  int i =0;
  while (i < numCities-1) {
    len += Length(LineSeg(city[i], city[i+1]));
    i++;
  }
  return len;
}


void DrawTour(void) {
/* To be completed for Part A */
  int i=0;
  while (i < numCities-1) {
    DrawLineSeg(LineSeg(city[i], city[i+1]));
    i++;
  }
}
    

int TrySwap(int i) {
/* To be completed for Part B. */
/* Most students will have used two calls to TourLength.  
 * I have a slightly more efficient method which compares value
 * before deciding to swap.  */
  double idiff = 0;
  point_t swap;
  if (i <0 || i>numCities-2) 
    return FALSE;
  else {
    if (i >0)
      idiff += Length(LineSeg(city[i-1],city[i+1])) - Length(LineSeg(city[i-1],city[i]));
    if (i <numCities-2)
      idiff += Length(LineSeg(city[i], city[i+2])) - Length(LineSeg(city[i+1], city[i+2]));
    if (idiff <0) {
      swap=city[i];
      city[i]=city[i+1];
      city[i+1]=swap;
      return TRUE;
    }
    else 
      return FALSE;
  }
}


void SwapHeuristic(void) 
/*  Repeatedly attempts to improve the current tour
 *  by swapping adjacent cities.
 */
{
  int changed = TRUE, i;

  while (changed) {
    changed = FALSE;
    for (i = 1;  i <= numCities - 2;  ++i) {
      if (TrySwap(i)) changed = TRUE;
    }
  }
}


int TryReverse(int i, int j) {
/* To be completed for Part C. */
  double idiff = 0;
  point_t swap;
  int k;
  if (i>j) {
    k=i;
    i=j;
    j=k;
  }
  if (i <0 || j>numCities-1) 
    return FALSE;
  else {
    if (i >0)
      idiff += Length(LineSeg(city[i-1],city[j])) - Length(LineSeg(city[i-1],city[i]));
    if (j <numCities-1)
      idiff += Length(LineSeg(city[i], city[j+1])) - Length(LineSeg(city[j], city[j+1]));
    if (idiff <0) {
      for (k=i; k <= (i+j)/2; k++) {
	swap=city[k];
	city[k]=city[i+j-k];
	city[i+j-k]=swap;
      }
      return TRUE;
    }
    else 
      return FALSE;
  }
}



void TwoOptHeuristic(void) 
/*  Repeatedly attempts to improve the current tour
 *  by performing ``2-opt'' moves..
 */
{
  int changed = TRUE, i, j;

  while (changed) {
    changed = FALSE;
    for (i = 0;  i <= numCities - 2;  ++i) {
      for (j = i + 1;  j <= numCities - 1;  ++j) {
        if (TryReverse(i, j)) changed = TRUE;
      }
    }  
  }
}


void GreedyHeuristic(void) {
/* To be completed for Part D. */
  point_t swap;
  double dist;
  int i, j, k;
  for(i=0; i < numCities-1; i++) {
    dist = Length(LineSeg(city[i], city[i+1]));
    k = i+1;
    for(j=i+2; j<numCities; j++) 
      if (Length(LineSeg(city[i], city[j])) < dist) {
	k=j;
	dist = Length(LineSeg(city[i], city[j]));
      }
    if (k != i+1) {
      swap = city[i+1];
      city[i+1] = city[k];
      city[k]=swap;
    }
  }    
} 


int main(void) {
  ReadCities();
  printf("Initial length of the tour is %f.\n", TourLength());
  SwapHeuristic();
  printf("After calling SwapHeuristic, length is %f.\n", TourLength());
  TwoOptHeuristic();
  printf("After calling TwoOptHeuristic, length is %f.\n", TourLength());
  GreedyHeuristic();
  printf("After calling GreedyHeuristic, length is %f.\n", TourLength());
  return EXIT_SUCCESS;
}

