#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#define MAXAIRPORTS 20
#define MAXFLIGHTS 1000

typedef enum {FALSE, TRUE} bool_t;

typedef struct {int day, month, year;} date_t; 
typedef struct {int hours, mins;} timed_t; 

typedef struct {
  char apt[4];
  char dest[4];
  char airline[4];
  date_t date;
  timed_t depart;
  timed_t arrive;
  float cost;
  int seats;
} flight_t;

typedef struct {
  int first, second;
} pair_t;

flight_t allflights[MAXFLIGHTS];
int numflights = 0;


bool_t sameDate(date_t date1, date_t date2)  {
/* BEGIN PART (a) */
  if ((date1.day == date2.day) && (date1.month == date2.month)) {
    if (date1.year == date2.year) {
      return TRUE;
    }
  }  
  return FALSE;
/* END PART (a) */
}


int cheapestDirectFlight(char* from, char* to, date_t date) {
/* BEGIN PART (b) */
  int i;
  int cheapI = -1;
  for (i=0; i< numflights; i++) {
    if (strcmp(allflights[i].apt, from) == 0) {
      if (strcmp(allflights[i].dest, to) == 0) {
	if (sameDate(allflights[i].date, date) && (allflights[i].seats>0)) {
	  if ((cheapI <0) || (allflights[i].cost< allflights[cheapI].cost)){
	    cheapI = i;
	  }
	}
      }
    }
  }
  return cheapI;
/* END PART (b) */
}

bool_t connectingPair(flight_t fst, flight_t snd) {
/* BEGIN PART (c) */
  int arr, dep;
  date_t ad, dd;
  if ((fst.seats == 0)|| (snd.seats == 0)) 
    return FALSE; 
  else if (!(strcmp(fst.dest, snd.apt)))  {
    arr = fst.arrive.hours*60 +fst.arrive.mins;
    dep = snd.depart.hours*60 +snd.depart.mins;
    if (sameDate(fst.date, snd.date) && (arr + 60 <= dep)) {
      return TRUE;
    }
    else if (!sameDate(fst.date, snd.date)) {
      ad = fst.date;
      dd = snd.date;
      if (ad.year < dd.year) {
	return TRUE;
      }
      else if ((ad.year == dd.year) && (ad.month < dd.month)) {
        return TRUE;
      }
      else if ((ad.year == dd.year) && (ad.month == dd.month) && (ad.day <dd.day)) {
	return TRUE;
      }
      else 
	return FALSE;
    }
  }
  return FALSE;
/* END PART (c) */
} 
  
pair_t bestTransferOption(char* from, char* to, date_t date) {
/* BEGIN PART (d) */
  int i, j;
  int fly1 = -1, fly2 = -1;
  float cost=-1;
  pair_t two;
  for (i=0; i<numflights; i++) {
    if(sameDate(allflights[i].date, date))  {
      if(!(strcmp(allflights[i].apt, from))) {
        for (j=0; j<numflights; j++) {
          if (!(strcmp(allflights[j].dest,to))) {
	    if (sameDate(allflights[j].date, date)) {
	      if (connectingPair(allflights[i], allflights[j])) {
		if (((allflights[i].cost+allflights[j].cost) <cost)||fly1==-1) {
		  fly1 = i;
		  fly2 = j;
                  cost = allflights[i].cost+allflights[j].cost;
		}
	      }
	    }
          }
        }
      }
    }
  }
  two.first=fly1;
  two.second = fly2;
  return two;
/* END PART (d) */
}

/* DO NOT EDIT THE IMPLEMENTATION OF initialiseDB.  This is 
 * provided to allow you to test your implementation against
 * the file 'flightList.txt' containing an example set of 
 * scheduled flights, available seats and prices.  */

bool_t initialiseDB (char* filename) {
  FILE* flights;
  int ok = 12, i=0;
  char apt1[4], apt2[4], carr[4];
  int day, month, year, hours, mins, hrs, mns, space;
  float price;
  flights = fopen(filename, "r");
  if (flights == NULL) {
    printf("%s is not a file in the current directory.\n", filename);
  }
  else {
    while ((ok == 12) && (i < MAXFLIGHTS)) {
      ok = fscanf(flights, "%s %s %s", apt1, apt2, carr);
      ok += fscanf(flights, "%d/%d/%d", &day, &month, &year);
      ok += fscanf(flights, "%d:%d", &hours, &mins);
      ok += fscanf(flights, "%d:%d", &hrs, &mns);
      ok += fscanf(flights, "%f %d", &price, &space);
      if (ok == 12) {
        strcpy(allflights[i].apt, apt1);
        strcpy(allflights[i].dest, apt2);
        strcpy(allflights[i].airline, carr);
        allflights[i].date.day = day;
        allflights[i].date.month = month;
        allflights[i].date.year = year;
        allflights[i].depart.hours = hours;
        allflights[i].depart.mins = mins;
        allflights[i].arrive.hours = hrs;
        allflights[i].arrive.mins = mns;
        allflights[i].cost = price;
        allflights[i].seats = space;
        i++;
      }
    }
  }
  if ((i == MAXFLIGHTS) && (fscanf(flights, "%s", apt1) >0)) {
    return FALSE;
  }
  else if ((i < MAXFLIGHTS) && (ok < 12) && (ok > 0)) {
    return FALSE;
  }
  else {
    numflights = i;
    return TRUE;
  }
}


int main(void) {
  /* CODE FOR TESTING "sameDate" */
  /* --------------------------- */
  date_t a, b;
  a.day=17;
  a.month=12;
  a.year = 2014;
  b.day=17;
  b.month=12;
  b.year=2013;
  printf("----------------------------------------------------------------------------\n");
  printf("sameDate(a,b) returns %d (should be 0) when called with initial vals for a,b.\n", sameDate(a,b)); 
  b.year=2014;
  printf("sameDate(a,b) returns %d (should be 1) when called with second vals of a,b.\n", sameDate(a,b));
  b.day=16;
  printf("sameDate(a,b) returns %d (should be 0) when called with third vals of a,b.\n", sameDate(a,b));
  b.day = 18;
 /* END OF CODE FOR TESTING "sameDate" */ 
 /* ---------------------------------- */
 if (initialiseDB("flightList2.txt")) {
   printf("--------------------------\n");
   printf("Initialisation successful.\n");
 }
  /* CODE FOR TESTING "cheapestDirectFlight" */
  /* --------------------------------------- */
  int t1 = cheapestDirectFlight("DUB", "EDI", b);
  printf("-------------------------------------------------------\n");
  if (t1==0) { 
    printf("Test1 of cheapestDirectFlight returns %d, this is correct.\n",t1);
  } else if (t1==-1) {
    printf("Test1 of cheapestDirectFlight returns %d, missed the flight in cell 0.\n",t1);
  } else {
    printf("Test1 of cheapestDirectFlight returns %d, only direct flight is cell 0.\n",t1);
  }
  t1 = cheapestDirectFlight("ORK", "EDI", a);
  if (t1==-1) {
    printf("Test2 of cheapestDirectFlight returns %d (no flight), this is correct.\n",t1);
  } else if (t1 == 9) {
    printf("Test2 of cheapestDirectFlight returns %d, this is incorrect.\n", t1);
    printf("    The dates don't match and the result should be -1 (no flight).\n");
  } else if (t1 > 16) {
    printf("Test2 of cheapestDirectFlight returns %d, this is incorrect.\n", t1);
    printf("    This cell does not store any flight (cells >= 17 are empty).\n");
  } else {
    printf("Test2 of cheapestDirectFlight returns %d, this is incorrect.\n", t1);
    printf("    Flight doesn't match ORK -> EDI.  Result should be -1 (no flight).\n");
  }
  t1 = cheapestDirectFlight("ORK", "EDI", b); 
  if (t1== 9) {
    printf("Test3 of cheapestDirectFlight returns %d, this is correct.\n",t1);
  } else if (t1== -1) {
    printf("Test3 of cheapestDirectFlight returns %d, missed the flight in cell 9.\n",t1);
  } else if (t1 >16) {
    printf("Test3 of cheapestDirectFlight returns %d, missed the flight in cell 9.\n", t1);
    printf("   This cell does not store any flight (cells >= 17 are empty).\n");
  } else {
    printf("Test3 of cheapestDirectFlight returns %d, should be 9.\n",t1);
    printf("   Flight in cell %d doesn't even match ORK -> EDI.\n", t1);
  } 
  t1 = cheapestDirectFlight("NCE", "LHR", b);
  if (t1== 7) { 
    printf("Test4 of cheapestDirectFlight returns %d, this is correct.\n",t1);
  } else if (t1 ==2) { 
    printf("Test4 of cheapestDirectFlight returns %d, incorrect.  The flight\n", t1);
    printf("    in cell 7 matches airports and date, and is cheaper.\n");
  } else if (t1 ==6) {
    printf("Test4 of cheapestDirectFlight returns %d, should be 7. The flight\n",t1);
    printf("    in cell 6 matches NCE -> LHR, but is on the wrong date.\n");
  } else if (t1 >16) {
    printf("Test4 of cheapestDirectFlight returns %d, should be 7.  In fact\n", t1);
    printf("    this cell does not store any flight (cells >= 17 are empty).\n");
  } else if (t1== -1) {
    printf("Test4 of cheapestDirectFlight returns %d, missed the flight in cell 7\n",t1);
    printf("    (and also missed another more-expensive NCE -> LHR option in cell 2).\n");
  } else { 
    printf("Test4 of cheapestDirectFlight returns %d, should be 7.\n",t1);
    printf("    Flight in cell %d doesn't even match NCE -> LHR.\n", t1);
  }
  t1 = cheapestDirectFlight("LHR", "DUB", b);
  if (t1 == 15) {
    printf("Test5 of cheapestDirectFlight returns %d, this is correct.\n",t1);
  } else if (t1 == 13) { 
    printf("Test5 of cheapestDirectFlight returns %d, should be 15.\n",t1);
    printf("    Cell 13 is a LHR -> DUB flight on the 18th, but it is full.\n");
  } else if (t1 > 16) {
    printf("Test5 of cheapestDirectFlight returns %d, should be 15.  In fact\n", t1);
    printf("    this cell does not store any flight (cells >= 17 are empty).\n");
  } else { 
    printf("Test5 of cheapestDirectFlight returns %d, should be 15.\n",t1);
    printf("    Flight in cell %d doesn't even match LHR -> DUB.\n", t1);
  }
  /* END OF CODE FOR TESTING "cheapestDirectFlight" */
  /* ---------------------------------------------- */
  /* CODE FOR TESTING "connectingPair" */
  /* --------------------------------------- */
  printf("------------------------------------------------\n");
  t1= connectingPair(allflights[12], allflights[4]);
  printf("Test1 of connectingPair returns %d, should be 0 (no common airport).\n",t1);
  t1= connectingPair(allflights[5], allflights[4]);
  printf("Test2 of connectingPair returns %d, should be 1.\n",t1);
  t1= connectingPair(allflights[2], allflights[4]);
  printf("Test3 of connectingPair returns %d, should be 1.\n",t1);
  t1= connectingPair(allflights[2], allflights[8]);
  printf("Test4 of connectingPair returns %d, should be 0.  This pair of flights\n", t1);
  printf("    connects through LHR, however 1st flight is 18th, 2nd flight is 17th).\n");
  t1= connectingPair(allflights[5], allflights[13]);
  printf("Test5 of connectingPair returns %d, should be 0 (second flight is full).\n",t1);
  t1= connectingPair(allflights[16], allflights[0]);
  printf("Test6 of connectingPair returns %d, should be 0 (only 55mins in-between).\n",t1);
  /* END OF CODE FOR TESTING "connectingPair" */
  /* ---------------------------------------- */
  /* CODE FOR TESTING "bestTransferOption" */
  /* ------------------------------------- */
  pair_t conn;
  conn = bestTransferOption("NCE", "EDI", a);
  printf("------------------------------------------------------------\n");
  printf("Test1 of bestTransferOptions returns %d, %d;   should be 6, 8.\n",conn.first, conn.second);
  conn = bestTransferOption("CDG", "EDI", b);
  printf("Test2 of bestTransferOptions returns %d, %d; should be 12, 0.\n",conn.first, conn.second);
  conn = bestTransferOption("LHR", "EDI", b);
  printf("Test3 of bestTransferOptions returns %d, %d; should be -1, -1.\n",conn.first, conn.second);
  conn = bestTransferOption("NCE", "EDI", b);
  printf("Test4 of bestTransferOptions returns %d, %d;  should be 3, 0.\n",conn.first, conn.second);
  conn = bestTransferOption("HAM", "DUB", b);
  printf("Test5 of bestTransferOptions returns %d, %d; should be 5, 15.\n",conn.first, conn.second);
  /* END OF CODE FOR TESTING "bestTransferOption" */
  /* -------------------------------------------- */
  return EXIT_SUCCESS;
}

